/**
 * Ollama provider implementation using the official Ollama npm package
 */
import { Ollama, } from 'ollama';
import { TokenUsage } from '../../tokens.js';
/**
 * Ollama provider implementation using the official Ollama npm package
 */
export class OllamaProvider {
    name = 'ollama';
    provider = 'ollama.chat';
    model;
    client;
    constructor(model, options = {}) {
        this.model = model;
        const baseUrl = options.baseUrl ||
            process.env.OLLAMA_BASE_URL ||
            'http://localhost:11434';
        this.client = new Ollama({ host: baseUrl });
    }
    /**
     * Generate text using Ollama API via the official npm package
     */
    async generateText(options) {
        const { messages, functions, temperature = 0.7, maxTokens, topP, frequencyPenalty, presencePenalty, stopSequences, } = options;
        // Format messages for Ollama API
        const formattedMessages = this.formatMessages(messages);
        // Prepare request options
        const requestOptions = {
            model: this.model,
            messages: formattedMessages,
            stream: false,
            options: {
                temperature: temperature,
                ...(topP !== undefined && { top_p: topP }),
                ...(frequencyPenalty !== undefined && {
                    frequency_penalty: frequencyPenalty,
                }),
                ...(presencePenalty !== undefined && {
                    presence_penalty: presencePenalty,
                }),
                ...(stopSequences &&
                    stopSequences.length > 0 && { stop: stopSequences }),
            },
        };
        // Add max_tokens if provided
        if (maxTokens !== undefined) {
            requestOptions.options = {
                ...requestOptions.options,
                num_predict: maxTokens,
            };
        }
        // Add functions/tools if provided
        if (functions && functions.length > 0) {
            requestOptions.tools = this.convertFunctionsToTools(functions);
        }
        // Make the API request using the Ollama client
        const response = await this.client.chat({
            ...requestOptions,
            stream: false,
        });
        // Extract content and tool calls
        const content = response.message?.content || '';
        // Handle tool calls if present
        const toolCalls = this.extractToolCalls(response);
        // Create token usage from response data
        const tokenUsage = new TokenUsage();
        tokenUsage.output = response.eval_count || 0;
        tokenUsage.input = response.prompt_eval_count || 0;
        return {
            text: content,
            toolCalls: toolCalls,
            tokenUsage: tokenUsage,
        };
    }
    /*
    interface Tool {
      type: string;
      function: {
          name: string;
          description: string;
          parameters: {
              type: string;
              required: string[];
              properties: {
                  [key: string]: {
                      type: string;
                      description: string;
                      enum?: string[];
                  };
              };
          };
      };
  }*/
    /**
     * Convert our function definitions to Ollama tool format
     */
    convertFunctionsToTools(functions) {
        return functions.map((fn) => ({
            type: 'function',
            function: {
                name: fn.name,
                description: fn.description,
                parameters: fn.parameters,
            },
        }));
    }
    /**
     * Extract tool calls from Ollama response
     */
    extractToolCalls(response) {
        if (!response.message?.tool_calls) {
            return [];
        }
        return response.message.tool_calls.map((toolCall) => {
            return {
                id: `tool-${Date.now()}-${Math.random().toString(36).substring(2, 11)}`,
                name: toolCall.function?.name,
                content: typeof toolCall.function?.arguments === 'string'
                    ? toolCall.function.arguments
                    : JSON.stringify(toolCall.function?.arguments || {}),
            };
        });
    }
    /**
     * Format messages for Ollama API
     */
    formatMessages(messages) {
        const output = [];
        messages.forEach((msg) => {
            switch (msg.role) {
                case 'user':
                case 'assistant':
                case 'system':
                    output.push({
                        role: msg.role,
                        content: msg.content,
                    });
                    break;
                case 'tool_result':
                    // Ollama expects tool results as a 'tool' role
                    output.push({
                        role: 'tool',
                        content: typeof msg.content === 'string'
                            ? msg.content
                            : JSON.stringify(msg.content),
                    });
                    break;
                case 'tool_use': {
                    // So there is an issue here is that ollama expects tool calls to be part of the assistant message
                    // get last message and add tool call to it
                    const lastMessage = output[output.length - 1];
                    lastMessage.tool_calls = [
                        {
                            function: {
                                name: msg.name,
                                arguments: JSON.parse(msg.content),
                            },
                        },
                    ];
                    break;
                }
            }
        });
        return output;
    }
}
//# sourceMappingURL=ollama.js.map